# -*- coding: utf-8 -*-
from django.template.context import RequestContext


class CmsRequestContext(RequestContext):
    """
    Version of the request context that maintains additional information for
    requests.

    """
    def __init__(self, site, request, resource, *args, **kwargs):
        self.site = site
        self.resource = resource
        super(CmsRequestContext, self).__init__(request, *args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.contrib.sites.shortcuts import get_current_site
from django.template import loader
from django.utils.safestring import mark_safe
from .context import CmsRequestContext


def render_resource(resource, request):
    """
    Render a resource.

    :param resource: Resource to render
    :param request: Current request object.

    """
    site = get_current_site(request)

    # Build up rendering context
    params = {r.code: mark_safe(r.value) for r in resource.fields.all()}
    params['title'] = resource.title

    context = CmsRequestContext(site, request, resource, params)

    # Identify and load template
    template = loader.select_template([
        "%s/%s" % (site.domain, resource.type.default_template),
        resource.type.default_template
    ])

    # Render
    return template.render(context)



# -*- coding: utf-8 -*-


class LazySettings(object):
    def _load_settings(self):
        from warthog import default_settings
        from django.conf import settings as django_settings

        for key in dir(default_settings):
            if not key.startswith('CMS_'):
                continue

            setattr(self, key, getattr(django_settings, key,
                getattr(default_settings, key)))

    def __getattr__(self, attr):
        self._load_settings()
        del self.__class__.__getattr__
        return self.__dict__[attr]
settings = LazySettings()



# -*- coding: utf-8 -*-
from django.core.cache import cache as default_cache


def generate_obj_key(instance_or_type, **vary_by):
    """Generate a cache key for a model object."""
    opts = instance_or_type._meta
    return 'model:%s.%s[%s]' % (
        opts.app_label, opts.module_name,
        ','.join(['%s=%s' % v for v in vary_by.iteritems()])
    )


def set_model(model_instance, cache=None):
    """
    Store a model in cache.
    
    :param model_instance: the model object to store.
    :param cache: cache instance to use; defaults to main django cache.
    :returns: cache key.
    
    """
    cache = cache or default_cache
    key = generate_obj_key(model_instance, pk=model_instance.pk)
    cache.set(key, model_instance)
    return key


def get_model(model_type, pk, cache=None):
    """Store a model in cache.
    
    :param model_type: model type for building cache key.
    :param pk: primary key of model to fetch from cache.
    :param cache: cache instance to use; defaults to main django cache.
    :returns: model object if found; else None.
    
    """
    cache = cache or default_cache
    key = generate_obj_key(model_type, pk=pk)
    return cache.get(key)


def clear_model(model_instance, cache=None):
    """Clear a model instance from cache.

    :param model_instance: the model object to store.
    :param cache: cache instance to use; defaults to main django cache.

    """
    cache = cache or default_cache
    key = generate_obj_key(model_instance, pk=model_instance.pk)
    return cache.delete(key)


def set_model_by_attribute(model_instance, attr_name, cache=None):
    """Store a model in cache by attribute value.
    
    :param model_instance: the model object to store.
    :param attr_name: attribute name.
    :param cache: cache instance to use; defaults to main django cache.
    :returns: reference cache key.
    
    .. note::
        Attribute must be unique.

    """
    cache = cache or default_cache
    # TODO: Add check for uniqueness (use unique flag)
    value = getattr(model_instance, attr_name)
    reference_key = generate_obj_key(model_instance, **{attr_name: value})
    key = set_model(model_instance, cache)
    cache.set(reference_key, key)
    return reference_key


def get_model_by_attribute(model_type, attr_name, value, cache=None):
    """Get a model from cache by reference.
    
    :param model_type: model type for building cache key.
    :param attr_name: attribute name.
    :param value: value of attribute.
    :param cache: cache instance to use; defaults to main django cache.
    :returns: model object if found; else None.

    """
    cache = cache or default_cache
    reference_key = generate_obj_key(model_type, **{attr_name: value})
    key = cache.get(reference_key)
    if key:
        return cache.get(key)
    else:
        return None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Resource',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(help_text='The name/title of the resource. Avoid using backslashes in the name.', max_length=100, verbose_name='title')),
                ('slug', models.CharField(help_text='Resource identifier used to create URI path.', max_length=100, null=True, verbose_name='slug')),
                ('uri_path', models.CharField(help_text='Path used in URI to find this resource.', max_length=500, verbose_name='resource path', db_index=True, blank=True)),
                ('published', models.BooleanField(default=False, help_text='The resource is published.', verbose_name='published')),
                ('publish_date', models.DateTimeField(help_text='Optional; if date is set this resource will go live once this date is reached.', null=True, verbose_name='go live date', blank=True)),
                ('unpublish_date', models.DateTimeField(help_text='Optional; if date is set this resource will expire once this date has passed.', null=True, verbose_name='expiry date', blank=True)),
                ('menu_title_raw', models.CharField(help_text='Optional name of page to display in menu entries. If not supplied the title field is used.', max_length=100, verbose_name='menu title', blank=True)),
                ('menu_class', models.CharField(help_text='Value to set template variable menuClass, to apply to menu items.', max_length=50, verbose_name='menu CSS class', blank=True)),
                ('hide_from_menu', models.BooleanField(default=False, help_text='Do not display this page in generated menus.', verbose_name='hide from navigation')),
                ('order', models.PositiveIntegerField(default=100, help_text='Ordering used to render element.', verbose_name='Order')),
                ('deleted', models.BooleanField(default=False, help_text='The resource should be treated as deleted and not displayed to the public at any time.', verbose_name='deleted')),
                ('edit_lock', models.BooleanField(default=False, help_text='Lock this resource so only `Can admin resources` permission can editthis resource. This flag is used to prevent structural resources from being modified.', verbose_name='Edit locked')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last modified')),
                ('parent', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, blank=True, to='warthog.Resource', null=True)),
                ('site', models.ForeignKey(default=1, to='sites.Site')),
            ],
            options={
                'ordering': ['order', 'uri_path', 'title'],
                'verbose_name': 'resource',
                'verbose_name_plural': 'resources',
                'permissions': (('preview_resource', 'Can preview resource'), ('admin_resource', 'Can admin resources.')),
            },
        ),
        migrations.CreateModel(
            name='ResourceField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('code', models.CharField(help_text='Code name used to reference this field.', max_length=50, verbose_name='code', validators=[django.core.validators.RegexValidator(b'^[-\\w]+$', message=b'Code value')])),
                ('value', models.TextField(null=True, blank=True)),
                ('resource', models.ForeignKey(related_name='fields', to='warthog.Resource')),
            ],
        ),
        migrations.CreateModel(
            name='ResourceType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=50, verbose_name='name')),
                ('code', models.CharField(help_text='Code name used to reference this field.', unique=True, max_length=50, verbose_name='code', validators=[django.core.validators.RegexValidator(b'^[-\\w]+$', message=b'Code value')])),
                ('description', models.CharField(help_text='Optional description of this resource type.', max_length=500, verbose_name='description', blank=True)),
                ('default_template', models.CharField(help_text='Name of the default template used to render resources of this type.', max_length=500, verbose_name='default template')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last modified')),
                ('child_types', models.ManyToManyField(help_text='Resource types that can be created as leaf nodes of the current resource.', to='warthog.ResourceType', verbose_name='Child types', blank=True)),
                ('site', models.ManyToManyField(default=[1], to='sites.Site')),
            ],
            options={
                'verbose_name': 'resource type',
                'verbose_name_plural': 'resource types',
            },
        ),
        migrations.CreateModel(
            name='ResourceTypeField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('code', models.CharField(help_text='Code name used to reference this field.', max_length=50, verbose_name='code', validators=[django.core.validators.RegexValidator(b'^[-\\w]+$', message=b'Code value')])),
                ('field_type', models.CharField(max_length=25, verbose_name='field type', choices=[(b'char', b'Text'), (b'html', b'HTML'), (b'bool', b'Checkbox'), (b'file', b'File'), (b'time', b'Time'), (b'date', b'Date'), (b'text', b'Text Area'), (b'image', b'Image'), (b'datetime', b'Date Time')])),
                ('required', models.BooleanField(default=False, help_text='This field is required', verbose_name='required')),
                ('label_raw', models.CharField(help_text='Label displayed to user.', max_length=200, null=True, verbose_name='label', blank=True)),
                ('help_text', models.CharField(help_text='Optional help message for describing the use of the field.', max_length=500, null=True, verbose_name='help text', blank=True)),
                ('resource_type', models.ForeignKey(related_name='fields', to='warthog.ResourceType')),
            ],
            options={
                'verbose_name': 'resource type field',
                'verbose_name_plural': 'resource type fields',
            },
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=100, verbose_name='name', db_index=True)),
                ('description', models.CharField(help_text='Optional description of template.', max_length=250, verbose_name='description', blank=True)),
                ('content', models.TextField(verbose_name='content')),
                ('mime_type', models.CharField(default=b'text/html', help_text='Mime-type to be set for this template.', max_length=25, verbose_name='MIME type', choices=[(b'Text (text/*)', ((b'text/html', 'HTML'), (b'text/plain', 'Text'), (b'text/css', 'CSS'), (b'text/javascript', 'JavaScript'), (b'text/csv', 'CSV'), (b'text/xml', 'XML'), (b'text/cachemanifest', 'HTML5 Cache Manifest'))), (b'Application (application/*)', ((b'application/xhtml+xml', 'XHTML'), (b'application/javascript', 'JavaScript'), (b'application/json', 'JSON')))])),
                ('cacheable', models.BooleanField(default=True, verbose_name='cachable')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last modified')),
                ('site', models.ManyToManyField(default=[1], to='sites.Site')),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'template',
                'verbose_name_plural': 'templates',
            },
        ),
        migrations.AddField(
            model_name='resource',
            name='type',
            field=models.ForeignKey(related_name='resources', to='warthog.ResourceType'),
        ),
        migrations.AlterUniqueTogether(
            name='resourcetypefield',
            unique_together=set([('resource_type', 'code')]),
        ),
        migrations.AlterUniqueTogether(
            name='resourcefield',
            unique_together=set([('resource', 'code')]),
        ),
        migrations.AlterUniqueTogether(
            name='resource',
            unique_together=set([('site', 'slug', 'parent')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.contrib.sites.models import Site
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as t
from . import resource_types
from .managers import CachingManager, ResourceManager, ResourceTypeManager


code_name = RegexValidator(r'^[-\w]+$', message='Code value')


class Template(models.Model):
    """Defines a template.

    A template is used as the scaffolding for a page (or section of a page).

    Variables can be used within a template to define additional data items
    to be used when rendering the page.

    """
    MIME_TYPES = (
        ('Text (text/*)', (
            ('text/html',          t('HTML')),
            ('text/plain',         t('Text')),
            ('text/css',           t('CSS')),
            ('text/javascript',    t('JavaScript')),
            ('text/csv',           t('CSV')),
            ('text/xml',           t('XML')),
            ('text/cachemanifest', t('HTML5 Cache Manifest')),
        )),
        ('Application (application/*)', (
            ('application/xhtml+xml',  t('XHTML')),
            ('application/javascript', t('JavaScript')),
            ('application/json',       t('JSON')),
        ))
    )

    # Description
    site = models.ManyToManyField(Site, default=[settings.SITE_ID])
    name = models.CharField(t('name'), max_length=100, db_index=True, unique=True)
    description = models.CharField(t('description'), max_length=250, blank=True,
        help_text=t("Optional description of template."))
    # Content
    content = models.TextField(t('content'))
    mime_type = models.CharField(t('MIME type'), choices=MIME_TYPES, default='text/html',
        max_length=25, help_text=t("Mime-type to be set for this template."))
    # Options
    cacheable = models.BooleanField(t('cachable'), default=True)
    # Tracking
    created = models.DateTimeField(t('creation date'), auto_now_add=True)
    updated = models.DateTimeField(t('last modified'), auto_now=True)

    class Meta:
        verbose_name = t('template')
        verbose_name_plural = t('templates')
        ordering = ['name', ]

    def __unicode__(self):
        return self.name


class ResourceType(models.Model):
    """
    Defines a resource and what fields that are associated with it.
    """
    site = models.ManyToManyField(Site, default=[settings.SITE_ID])
    name = models.CharField(
        verbose_name=t('name'),
        max_length=50, unique=True
    )
    code = models.CharField(
        verbose_name=t('code'),
        max_length=50, unique=True, validators=[code_name],
        help_text=t('Code name used to reference this field.')
    )
    description = models.CharField(
        verbose_name=t('description'),
        max_length=500, blank=True,
        help_text=t('Optional description of this resource type.')
    )
    default_template = models.CharField(
        verbose_name=t('default template'),
        max_length=500,
        help_text=t('Name of the default template used to render resources of this type.')
    )
    child_types = models.ManyToManyField(
        to='self',
        verbose_name=t('Child types'),
        symmetrical=False, blank=True,
        help_text=t('Resource types that can be created as leaf nodes of the current resource.')
    )
    created = models.DateTimeField(
        verbose_name=t('creation date'),
        auto_now_add=True
    )
    updated = models.DateTimeField(
        verbose_name=t('last modified'),
        auto_now=True
    )

    objects = ResourceTypeManager()

    class Meta:
        verbose_name = t('resource type')
        verbose_name_plural = t('resource types')

    def __unicode__(self):
        return self.name


class ResourceTypeField(models.Model):
    """
    Defines a field that is part of a resource.
    """
    resource_type = models.ForeignKey(ResourceType, related_name='fields')
    code = models.CharField(
        verbose_name=t('code'),
        max_length=50, validators=[code_name],
        help_text=t('Code name used to reference this field.')
    )
    field_type = models.CharField(
        verbose_name=t('field type'),
        max_length=25,
        choices=resource_types.library.choices
    )
    required = models.BooleanField(
        verbose_name=t('required'),
        default=False,
        help_text=t('This field is required')
    )
    label_raw = models.CharField(
        verbose_name=t('label'),
        max_length=200, null=True, blank=True,
        help_text=t('Label displayed to user.')
    )
    help_text = models.CharField(
        verbose_name=t('help text'),
        max_length=500, null=True, blank=True,
        help_text=t('Optional help message for describing the use of the field.')
    )

    objects = CachingManager()

    class Meta:
        verbose_name = t('resource type field')
        verbose_name_plural = t('resource type fields')
        unique_together = (('resource_type', 'code'), )

    def __unicode__(self):
        return self.code

    @property
    def label(self):
        return self.label_raw if self.label_raw else self.code

    def get_kwargs(self):
        return {
            'label': self.label,
            'required': self.required,
            'help_text': self.help_text,
        }

    def create_form_field(self):
        """
        Create an instance of a form field that that is represented by this model.
        """
        return resource_types.library[self.field_type].create_form_field(self.get_kwargs())


class Resource(models.Model):
    """
    CMS Resource model, resource that is served on a particular URI.
    """

    CONTENT_DISPOSITIONS = (
        ('',           t('None')),
        ('inline',     t('Inline')),
        ('attachment', t('Attachment'))
    )

    # Statuses of a resource
    STATUS_DELETED = 0
    STATUS_UNPUBLISHED = 1
    STATUS_EXPIRED = 2
    STATUS_SCHEDULED = 3
    STATUS_LIVE = 4

    STATUS_EXPANDED = {
        STATUS_DELETED: ('deleted', t('Deleted'), t('This resources has been deleted.')),
        STATUS_UNPUBLISHED: ('unpublished', t('Unpublished'), t('This resource has not been published.')),
        STATUS_EXPIRED: ('expired', t('Expired'), t('This resource has expired (un-publish date is in the past).')),
        STATUS_SCHEDULED: ('scheduled', t('Scheduled'), t('This resource is scheduled to be published (publish date is in the future).')),
        STATUS_LIVE: ('live', t('Live'), t('This resource is live.')),
    }

    site = models.ForeignKey(Site, default=settings.SITE_ID)
    type = models.ForeignKey(ResourceType, related_name=t('resources'))
    title = models.CharField(
        verbose_name=t('title'),
        max_length=100,
        help_text=t("The name/title of the resource. Avoid using backslashes in the name.")
    )
    slug = models.CharField(
        verbose_name=t('slug'),
        max_length=100,
        null=True,
        help_text=t('Resource identifier used to create URI path.')
    )
    uri_path = models.CharField(
        verbose_name=t('resource path'),
        max_length=500, db_index=True,
        blank=True,
        help_text=t("Path used in URI to find this resource.")
    )
    published = models.BooleanField(
        verbose_name=t('published'),
        default=False,
        help_text=t("The resource is published.")
    )
    publish_date = models.DateTimeField(
        verbose_name=t('go live date'),
        null=True, blank=True,
        help_text=t("Optional; if date is set this resource will go live once this date is reached.")
    )
    unpublish_date = models.DateTimeField(
        verbose_name=t('expiry date'),
        null=True, blank=True,
        help_text=t("Optional; if date is set this resource will expire once this date has passed.")
    )
    # Menu
    parent = models.ForeignKey(
        to='self',
        null=True, blank=True,
        on_delete=models.PROTECT
    )
    menu_title_raw = models.CharField(
        verbose_name=t('menu title'),
        max_length=100, blank=True,
        help_text=t("Optional name of page to display in menu entries. If not supplied the title field is used.")
    )
    menu_class = models.CharField(
        verbose_name=t('menu CSS class'),
        max_length=50, blank=True,
        help_text=t("Value to set template variable menuClass, to apply to menu items.")
    )
    hide_from_menu = models.BooleanField(
        verbose_name=t('hide from navigation'),
        default=False,
        help_text=t("Do not display this page in generated menus.")
    )
    # Flags
    order = models.PositiveIntegerField(
        verbose_name=t('Order'),
        default=100,
        help_text=t('Ordering used to render element.')
    )
    deleted = models.BooleanField(
        verbose_name=t('deleted'),
        default=False,
        help_text=t("The resource should be treated as deleted and not displayed to the public at any time.")
    )
    edit_lock = models.BooleanField(
        verbose_name=t('Edit locked'),
        default=False,
        help_text=t(
            "Lock this resource so only `Can admin resources` permission can edit"
            "this resource. This flag is used to prevent structural resources from being modified."
        )
    )

    # Details
    created = models.DateTimeField(t('creation date'), auto_now_add=True)
    updated = models.DateTimeField(t('last modified'), auto_now=True)

    objects = ResourceManager()

    class Meta:
        verbose_name = t('resource')
        verbose_name_plural = t('resources')
        permissions = (
            ('preview_resource', 'Can preview resource'),
            ('admin_resource', 'Can admin resources.'),
        )
        ordering = ['order', 'uri_path', 'title', ]
        unique_together = (('site', 'slug', 'parent', ), )

    def __unicode__(self):
        return '[%s] - %s (%s)' % (
            self.title, Resource.STATUS_EXPANDED[self.published_status][1], self.uri_path)

    @models.permalink
    def get_absolute_url(self):
        return 'warthog-preview', [str(self.pk)]

    def clean(self):
        # Ensure dates are valid
        if (self.publish_date is not None) and \
           (self.unpublish_date is not None) and \
           (self.publish_date > self.unpublish_date):
            raise ValidationError('Publish date must be prior to the Un-publish date.')

    @property
    def children(self):
        return self.resource_set.filter_front()

    @property
    def published_status(self):
        """Determine publish status of this resource."""
        if self.deleted:
            return self.STATUS_DELETED
        if self.published:
            now = timezone.now()
            if (self.publish_date is not None) and now < self.publish_date:
                return self.STATUS_SCHEDULED
            elif (self.unpublish_date is not None) and now > self.unpublish_date:
                return self.STATUS_EXPIRED
            else:
                return self.STATUS_LIVE
        return self.STATUS_UNPUBLISHED

    @property
    def is_live(self):
        """Is this model considered live."""
        return self.published_status == self.STATUS_LIVE

    @property
    def is_root(self):
        """Is this model at the root."""
        return not bool(self.parent)

    @property
    def menu_title(self):
        """Title to appear in any menus (uses title if menu_title_raw is
        bool(False))."""
        return self.menu_title_raw or self.title

    def is_locked_for_user(self, user):
        """Check if a particular user can edit this resource"""
        if self.edit_lock:
            return not user.has_perm('warthog.admin_resource')
        return False

    def can_serve(self, user, allow_superuser=True, permission='preview_resource'):
        """Determine if this resource can be served.

        :param user: User we are checking against.
        :param allow_superuser: Allow superusers to view this resource.
        :param permission: Permission required to view this resource.

        """
        if self.is_live:
            return True

        if allow_superuser and user.is_superuser:
            return True

        if permission and user.has_perm(permission):
            return True

        return False


class ResourceField(models.Model):
    """Template variable that is applied to a resource."""
    resource = models.ForeignKey(Resource, related_name='fields')
    code = models.CharField(
        verbose_name=t('code'),
        max_length=50, validators=[code_name],
        help_text=t('Code name used to reference this field.')
    )
    value = models.TextField(
        blank=True, null=True
    )

    class Meta:
        unique_together = (('resource', 'code',), )

    def __unicode__(self):
        return "%s=%s" % (self.code, self.value)



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models.query import QuerySet
from django.conf import settings
from django.core.cache import cache


def generate_cache_key(instance_or_type, **vary_by):
    """Generate a cache key for a model object."""
    opts = instance_or_type._meta
    return 'model:{}[{}]'.format(
        opts.db_table,
        ','.join(['%s=%s' % v for v in vary_by.iteritems()])
    )


class CachingManager(models.Manager):
    """Manager that handles caching transparently."""
    def get_queryset(self):
        return CachingQuerySet(self.model)

    def contribute_to_class(self, model, name):
        models.signals.post_save.connect(self._post_save, sender=model)
        models.signals.post_delete.connect(self._post_save, sender=model)
        setattr(model, 'generate_cache_key', classmethod(generate_cache_key))
        setattr(model, 'cache_key', property(lambda self: self.generate_cache_key(pk=self.pk)))
        return super(CachingManager, self).contribute_to_class(model, name)

    def _invalidate_cache(self, instance):
        """
        Explicitly set a None value instead of just deleting so we don't have any race
        conditions where:
            Thread 1 -> Cache miss, get object from DB
            Thread 2 -> Object saved, deleted from cache
            Thread 1 -> Store (stale) object fetched from DB in cache
        Five second should be more than enough time to prevent this from happening for
        a web app.
        """
        cache.set(instance.cache_key, None, 5)

    def _post_save(self, instance, **kwargs):
        self._invalidate_cache(instance)

    def _post_delete(self, instance, **kwargs):
        self._invalidate_cache(instance)


class CachingQuerySet(QuerySet):
    def iterator(self):
        super_iterator = super(CachingQuerySet, self).iterator()
        while True:
            obj = super_iterator.next()
            # Use cache.add instead of cache.set to prevent race conditions (see CachingManager)
            cache.add(obj.cache_key, obj)
            yield obj

    def get(self, *args, **kwargs):
        """
        Checks the cache to see if there's a cached entry for this pk. If not, fetches
        using super then stores the result in cache.

        Most of the logic here was gathered from a careful reading of
        ``django.db.models.sql.query.add_filter``
        """
        # Punt on anything more complicated than get by pk/id only...
        # If there is any other ``where`` filter on this QuerySet just call
        # super. There will be a where clause if this QuerySet has already
        # been filtered/cloned.
        if not self.query.where and len(kwargs) == 1:
            k = kwargs.keys()[0]
            if k in ('pk', 'pk__exact', '%s' % self.model._meta.pk.attname,
                     '%s__exact' % self.model._meta.pk.attname):
                obj = cache.get(self.model.generate_cache_key(pk=kwargs[k]))
                if obj is not None:
                    obj.from_cache = True
                    return obj

        # Calls self.iterator to fetch objects, storing object in cache.
        return super(CachingQuerySet, self).get(*args, **kwargs)


class ResourceTypeManager(CachingManager):
    """Manager for resource type objects"""
    def get_queryset(self):
        return super(ResourceTypeManager, self).get_queryset()


class ResourceManager(CachingManager):
    """Manager for dealing with resource models."""
    use_for_related_fields = True

    def get_queryset(self):
        return super(ResourceManager, self).get_queryset()

    def get_front(self, **filters):
        """
        Apply default filters for getting an item for front display.

        By default applies published=True, delete=False, site=settings.SITE_ID

        :param filters: additional filters to apply
        :return: Resource
        """
        filters.update(published=True, deleted=False, site=settings.SITE_ID)
        return self.get(**filters)

    def filter_front(self, **filters):
        """
        Apply default filters for front display of items.

        By default applies published=True, delete=False, site=settings.SITE_ID

        :param filters: additional filters to apply
        :return: Queryset
        """
        filters.update(published=True, deleted=False, site=settings.SITE_ID)
        return self.filter(**filters)

    def get_uri_path(self, uri_path):
        """
        Get a resource from the URI path.

        :param uri_path: path to search for.

        """
        # Normalise path
        if len(uri_path) > 1 and uri_path.endswith('/'):
            uri_path = uri_path[:-1]

        # Try to get from cache
        ref_key = generate_cache_key(self.model, uri_path=uri_path)
        cache_key = cache.get(ref_key) if ref_key else None
        resource = cache.get(cache_key) if cache_key else None

        if not resource:
            resource = self.get_front(uri_path__exact=uri_path)

            cache_key = generate_cache_key(self.model, pk=resource.pk)
            cache.set(cache_key, resource)
            cache.set(ref_key, cache_key)
        return resource



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.conf import settings
from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.db import transaction
from django.forms.utils import ErrorList
from django.contrib.admin import helpers
from django.http import Http404
from django.utils.encoding import force_unicode
from django.utils.html import escape
from django.utils.translation import ugettext_lazy as _
from urlparse import unquote

from .forms import ResourceFieldsForm, ResourceAddForm
from .. import cache
from ..models import Template, ResourceType, ResourceTypeField, Resource


class CachedModelAdmin(admin.ModelAdmin):
    """Model admin class with built in cache clear actions"""
    actions = 'clear_cache'

    def clear_cache(self, request, queryset):
        map(cache.clear_model, queryset)
        count = len(queryset)
        if count > 1:
            message = '%s objects where' % count
        else:
            message = '1 object was'
        self.message_user(request, "%s invalidated from cache." % message)
    clear_cache.short_description = 'Invalidate selected objects in cache'


class TemplateAdmin(admin.ModelAdmin):
    """
    Admin class for Template model.
    """
    fieldsets = [
        (None, {
            'fields': ('site', 'name', 'description', 'cacheable', ) ,
        }),
        ('Content', {
            'fields': ('mime_type', 'content', ),
            'classes': ('wide', 'monospace', ),
        }),
        ('Details', {
            'fields': ('created', 'updated', ),
        }),
    ]
    list_display = ('name', 'description', 'created', 'updated', )
    list_display_links = ('name', )
    readonly_fields = ('created', 'updated', )
    save_on_top = True
    save_as = True

    def get_queryset(self, request):
        """
        Returns a QuerySet of all model instances that can be edited by the
        admin site. This is used by changelist_view.
        """
        qs = super(TemplateAdmin, self).get_queryset(request)
        if request.user.is_superuser and 'all' in request.GET:
            return qs
        return qs.filter(site=settings.SITE_ID)

admin.site.register(Template, TemplateAdmin)


class ResourceTypeFieldInline(admin.TabularInline):
    model = ResourceTypeField
    extra = 1


class ResourceTypeAdmin(admin.ModelAdmin):
    inlines = (ResourceTypeFieldInline, )
    list_display = ('name', 'description', 'child_types_list', 'created', 'updated', )
    list_display_superuser = ('name', 'site_summary', 'description', 'child_types_list', 'created', 'updated', )
    prepopulated_fields = {'code': ('name', )}
    save_on_top = True
    save_as = True
    filter_horizontal = ('child_types',)

    def site_summary(self, obj):
        """Summary of sites."""
        return ', '.join(s.name for s in obj.site.all())
    site_summary.short_description = _('Sites')

    def child_types_list(self, obj):
        return ', '.join(str(c.name) for c in obj.child_types.all())
    child_types_list.short_description = _('child types')

    def get_list_display(self, request):
        if request.user.is_superuser:
            return self.list_display_superuser
        else:
            return self.list_display

    def get_queryset(self, request):
        """
        Returns a QuerySet of all model instances that can be edited by the
        admin site. This is used by changelist_view.
        """
        qs = super(ResourceTypeAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(site=settings.SITE_ID)

admin.site.register(ResourceType, ResourceTypeAdmin)


class ResourceErrorList(ErrorList):
    """
    Stores all errors for the form/formsets in an add/change stage view.
    """
    def __init__(self, form, fields_form):
        super(ResourceErrorList, self).__init__()
        if form.is_bound:
            self.extend(form.errors.values())
        if fields_form.is_bound:
            self.extend(fields_form.errors.values())


class ResourceAdmin(CachedModelAdmin):
    """
    Admin class for Resource model.
    """
    fieldsets = [
        (None, {
            'fields': ('title', 'slug', 'uri_path', ('published', 'hide_from_menu'),
                       ('publish_date', 'unpublish_date'),),
        }),
        ('Details', {
            'fields': (('created', 'updated'),),
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': ('site', 'parent', 'menu_title_raw', 'menu_class', 'order', 'edit_lock', ),
        }),
    ]
    add_fieldsets = [
        (None, {
            'fields': ('site', 'type', 'title', 'slug', 'parent', )
        }),
        ('Hidden', {
            'classes': ('hidden',),
            'fields': ('order', )
        }),
    ]
    list_display = ('html_status', 'title', 'order', 'resource_path', 'html_type', 'published', 'publish_summary',
                    'unpublish_summary', 'html_actions', )
    list_display_superuser = ('html_status', 'site', 'order', 'title', 'resource_path', 'html_type', 'published',
                              'publish_summary', 'unpublish_summary', 'html_actions', )
    list_editable = ('order', )
    list_display_links = ('title', )
    list_filter = ('published', 'deleted', 'type', )
    actions = ('make_published', 'make_unpublished', 'clear_cache',)
    prepopulated_fields = {'slug': ('title', )}
    readonly_fields = ('created', 'updated', 'uri_path', 'edit_lock', 'site',)
    readonly_fields_superuser = ('created', 'updated', )
    save_on_top = True
    save_as = False

    def html_status(self, obj):
        """HTML representation of the status (primarily for use in Admin)."""
        code, name, help_text = Resource.STATUS_EXPANDED[obj.published_status]
        return '<span class="warthog-status warthog-status-%s" title="%s">%s</span>' % (
            code, unicode(help_text), unicode(name) + (', hidden' if obj.hide_from_menu else ''))
    html_status.short_description = _('status')
    html_status.allow_tags = True

    def resource_path(self, obj):
        """ Path to resource """
        return '<a href="%s">%s</a>' % (obj.get_absolute_url(), obj.uri_path)
    resource_path.short_description = _('Resource Path')
    resource_path.allow_tags = True

    def html_actions(self, obj):
        """Actions within the list"""
        child_types = obj.type.child_types.all()
        add_uri = reverse('admin:warthog_resource_add')
        actions = []
        for type in child_types:
            actions.append('<li><a href="%s?parent=%s&type=%s">Add %s resource</a></li>' % (
                add_uri, obj.pk, type.pk, type.name))
        return '<ul>%s</ul>' % ''.join(actions)
    html_actions.short_description = _('Actions')
    html_actions.allow_tags = True

    def html_type(self, obj):
        """HTML representation of the status (primarily for use in Admin)."""
        return '<span title="%s">%s</span>' % (obj.type.description, obj.type)
    html_type.short_description = _('type')
    html_type.allow_tags = True

    def publish_summary(self, obj):
        """Summary of publish dates."""
        if obj.publish_date:
            return str(obj.publish_date)
        else:
            return _('immediately')
    publish_summary.short_description = _('go live')

    def unpublish_summary(self, obj):
        """Summary of un-publish dates."""
        if obj.unpublish_date:
            return str(obj.unpublish_date)
        else:
            return _('never')
    unpublish_summary.short_description = _('expire ')

    def make_published(self, request, queryset):
        rows_updated = queryset.update(published=True)
        if rows_updated == 1:
            message_bit = "1 resource was"
        else:
            message_bit = "%s resources were" % rows_updated
        self.message_user(request, "%s successfully published." % message_bit)
    make_published.short_description = _('Publish selected resources')

    def make_unpublished(self, request, queryset):
        rows_updated = queryset.update(published=False)
        if rows_updated == 1:
            message_bit = "1 resource was"
        else:
            message_bit = "%s resources were" % rows_updated
        self.message_user(request, "%s successfully un-published." % message_bit)
    make_unpublished.short_description = _('Un-publish selected resources')

    def save_model(self, request, obj, form, change):
        import posixpath
        if not obj.uri_path:
            if obj.parent:
                obj.uri_path = posixpath.join(obj.parent.uri_path, obj.slug)
            else:
                obj.uri_path = '/'
        obj.save()

    def get_list_display(self, request):
        if request.user.is_superuser and 'all' in request.GET:
            return self.list_display_superuser
        else:
            return self.list_display

    def get_readonly_fields(self, request, obj=None):
        if request.user.has_perm('warthog.admin_resource'):
            return self.readonly_fields_superuser
        else:
            return self.readonly_fields

    def get_fieldsets(self, request, obj=None):
        if not obj:
            return self.add_fieldsets
        return super(ResourceAdmin, self).get_fieldsets(request, obj)

    def get_form(self, request, obj=None, **kwargs):
        if not obj:
            return ResourceAddForm
        return super(ResourceAdmin, self).get_form(request, obj, **kwargs)

    def get_queryset(self, request):
        """
        Returns a QuerySet of all model instances that can be edited by the
        admin site. This is used by changelist_view.
        """
        qs = super(ResourceAdmin, self).get_queryset(request)
        if request.user.is_superuser and 'all' in request.GET:
            return qs
        return qs.filter(site=settings.SITE_ID)

    def response_add(self, request, obj, post_url_continue=None):
        if '_addanother' not in request.POST and '_popup' not in request.POST:
            request.POST['_continue'] = 1
        return super(ResourceAdmin, self).response_add(request, obj, post_url_continue)

    def save_fields_form(self, request, form, obj, change):
        """
        Given a ModelForm return an unsaved instance. ``change`` is True if
        the object is being changed, and False if it's being added.
        """
        return form.save_to(obj)

    def has_change_permission(self, request, obj=None):
        if obj and obj.is_locked_for_user(request.user):
            return False
        return super(ResourceAdmin, self).has_change_permission(request, obj)

    def has_delete_permission(self, request, obj=None):
        if obj and obj.is_locked_for_user(request.user):
            return False
        return super(ResourceAdmin, self).has_delete_permission(request, obj)

    @transaction.atomic
    def change_view(self, request, object_id, form_url='', extra_context=None):
        model = self.model
        opts = model._meta

        obj = self.get_object(request, unquote(object_id))
        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {
                'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})

#        if request.method == 'POST' and "_saveasnew" in request.POST:
#            return self.add_view(request, form_url=reverse('admin:%s_%s_create' %
#                                                           (opts.app_label, opts.module_name),
#                current_app=self.admin_site.name), resource_type_code=obj.type)

        ModelForm = self.get_form(request, obj)
        initial=dict(obj.fields.all().values_list('code', 'value'))
        if request.method == 'POST':
            form = ModelForm(data=request.POST, instance=obj)
            fields_form = ResourceFieldsForm(obj.type, instance=obj, initial=initial, prefix='fields', data=request.POST, files=request.FILES)

            if form.is_valid() and fields_form.is_valid():
                obj = self.save_form(request, form, change=False)
                self.save_model(request, obj, form, False)
                self.save_fields_form(request, fields_form, obj, False)
                self.log_addition(request, obj)
                return self.response_change(request, obj)
        else:
            form = ModelForm(instance=obj)

            # Populate form
            fields_form = ResourceFieldsForm(obj.type, instance=obj, initial=initial, prefix='fields')

        adminForm = helpers.AdminForm(form, list(self.get_fieldsets(request, obj)),
            self.get_prepopulated_fields(request, obj),
            self.get_readonly_fields(request, obj),
            model_admin=self)
        media = self.media + adminForm.media + fields_form.media

        context = {
            'title': _('Change %s %s') % (force_unicode(obj.type.name), force_unicode(opts.verbose_name)),
            'adminform': adminForm,
            'fields_adminform': fields_form,
            'object_id': object_id,
            'original': obj,
            'is_popup': "_popup" in request.REQUEST,
            'show_delete': True,
            'media': media,
            'inline_admin_formsets': [],
            'errors': ResourceErrorList(form, fields_form),
            'app_label': opts.app_label,
        }
        return self.render_change_form(request, context, change=True, obj=obj, form_url=form_url)

admin.site.register(Resource, ResourceAdmin)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.http import Http404
from .views import Cms


class CmsMiddleware(object):
    """
    Middleware for capturing unhandled URI's and loading matching content.

    """
    def __init__(self):
        self.view = Cms.as_view()

    def process_response(self, request, response):
        """
        Handle response event.

        :param request: object.
        :param response: object.

        """
        # No need to check for a cms resource for non-404 responses.
        if response.status_code == 404:
            try:
                return self.view(request)
            except Http404:
                pass
        return response



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django import template
from ..models import Resource, ResourceType
from ..data_structures import ResourceIterator, ResourceItem

register = template.Library()


@register.assignment_tag
def get_resource(pk_or_path):
    """
    Get a resource from it's ID or path.
    """
    try:
        filters = dict(pk=int(pk_or_path))
    except ValueError:
        # Assume is path
        filters = dict(uri_path=pk_or_path)

    try:
        resource = Resource.objects.get_front(**filters)
    except Resource.DoesNotExist:
        return None
    else:
        if resource.is_live:
            return ResourceItem(resource)


@register.assignment_tag
def get_resource_type(code, include_hidden=False):
    resource_type = ResourceType.objects.get(code=code)
    return ResourceIterator.for_type(resource_type, include_hidden)


@register.assignment_tag(takes_context=True)
def get_children(context, resource=None, include_hidden=False):
    if isinstance(resource, int):
        resource = get_resource(resource)
    elif resource is None:
        resource = context.resource
    return ResourceIterator.for_children(resource, include_hidden)



import datetime
from django import test
from django.core.exceptions import ValidationError
from django.utils import timezone
from warthog.models import Resource


THIRTY_MINUTES = datetime.timedelta(minutes=30)
FUTURE = timezone.now() + THIRTY_MINUTES
PAST = timezone.now() - THIRTY_MINUTES

TEST_RESOURCES = {
    'deleted': Resource(deleted=True, published=True, publish_date=None, unpublish_date=None),
    'live': Resource(deleted=False, published=True, publish_date=None, unpublish_date=None),
    'unpublished': Resource(deleted=False, published=False, publish_date=None, unpublish_date=None),
    'scheduled': Resource(deleted=False, published=True, publish_date=FUTURE, unpublish_date=None),
    'expired': Resource(deleted=False, published=True, publish_date=None, unpublish_date=PAST),
    'live_in_range': Resource(deleted=False, published=True, publish_date=PAST, unpublish_date=FUTURE),
    'live_no_start': Resource(deleted=False, published=True, publish_date=None, unpublish_date=FUTURE),
    'live_no_end': Resource(deleted=False, published=True, publish_date=PAST, unpublish_date=None),
}

PUBLISHED_STATUS = lambda t: t.published_status
IS_LIVE = lambda t: t.is_live


class ContentModelTestCase(test.TestCase):
    """
    Test cases for the content model.
    """
    def assertTargetEqual(self, target_resource, resolver, expected_value):
        target = TEST_RESOURCES[target_resource]
        actual_value = resolver(target)
        self.assertEqual(expected_value, actual_value)

    ## Published Status tests

    def test_published_status_deleted(self):
        self.assertTargetEqual('deleted', PUBLISHED_STATUS, Resource.STATUS_DELETED)

    def test_published_status_live(self):
        self.assertTargetEqual('live', PUBLISHED_STATUS, Resource.STATUS_LIVE)

    def test_published_status_unpublished(self):
        self.assertTargetEqual('unpublished', PUBLISHED_STATUS, Resource.STATUS_UNPUBLISHED)

    def test_published_status_scheduled(self):
        self.assertTargetEqual('scheduled', PUBLISHED_STATUS, Resource.STATUS_SCHEDULED)

    def test_published_status_expired(self):
        self.assertTargetEqual('expired', PUBLISHED_STATUS, Resource.STATUS_EXPIRED)

    def test_published_status_live_in_range(self):
        self.assertTargetEqual('live_in_range', PUBLISHED_STATUS, Resource.STATUS_LIVE)

    def test_published_status_live_no_start(self):
        self.assertTargetEqual('live_no_start', PUBLISHED_STATUS, Resource.STATUS_LIVE)

    def test_published_status_live_no_end(self):
        self.assertTargetEqual('live_no_end', PUBLISHED_STATUS, Resource.STATUS_LIVE)

    ## Is Live tests

    def test_is_live_deleted(self):
        self.assertTargetEqual('deleted', IS_LIVE, False)

    def test_is_live_live(self):
        self.assertTargetEqual('live', IS_LIVE, True)

    def test_is_live_unpublished(self):
        self.assertTargetEqual('unpublished', IS_LIVE, False)

    def test_is_live_scheduled(self):
        self.assertTargetEqual('scheduled', IS_LIVE, False)

    def test_is_live_expired(self):
        self.assertTargetEqual('expired', IS_LIVE, False)

    def test_is_live_live_in_range(self):
        self.assertTargetEqual('live_in_range', IS_LIVE, True)

    def test_is_live_live_no_start(self):
        self.assertTargetEqual('live_no_start', IS_LIVE, True)

    def test_is_live_live_no_end(self):
        self.assertTargetEqual('live_no_end', IS_LIVE, True)


    ## Other property tests

    def test_is_root(self):
        target = Resource(parent=None)
        self.assertTrue(target.is_root)

        parent = Resource()
        target = Resource(parent=parent)
        self.assertFalse(target.is_root)

    def test_menu_title(self):
        target = Resource(title="Foo")
        self.assertEqual("Foo", target.menu_title)

        target = Resource(title="Foo", menu_title_raw="Bar")
        self.assertEqual("Bar", target.menu_title)


    ## Methods

    def test_clean_no_dates(self):
        target = Resource()
        target.clean()

    def test_clean_no_publish_date(self):
        target = Resource(publish_date=None, unpublish_date=FUTURE)
        target.clean()

    def test_clean_no_unpublish_date(self):
        target = Resource(publish_date=PAST, unpublish_date=None)
        target.clean()

    def test_clean_valid_range(self):
        target = Resource(publish_date=PAST, unpublish_date=FUTURE)
        target.clean()

    def test_clean_invalid_range(self):
        target = Resource(publish_date=FUTURE, unpublish_date=PAST)
        self.assertRaises(ValidationError, lambda: target.clean())




# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django import forms
from ..models import Resource
from ..resource_types import library


class ResourceFieldsForm(forms.Form):
    """
    Form that dynamically creates form fields based on a ResourceType definition.
    """
    def __init__(self, resource_type, instance, *args, **kwargs):
        self.resource_type = resource_type
        self.instance = instance
        super(ResourceFieldsForm, self).__init__(*args, **kwargs)

        initial = kwargs.get('initial', {})
        for f in resource_type.fields.all():
            self.fields[f.code] = f.create_form_field()
            initial[f.code] = library[f.field_type].to_python(initial.get(f.code), instance.pk, f.code)
        self.initial = initial

    def save_to(self, obj):
        """
        Save the values on this form to a resource_types object.
        :param obj: ResourceObject to save form fields to.
        """
        changed_data = self.changed_data
        cleaned_data = self.cleaned_data
        obj.fields.filter(code__in=changed_data).delete()
        for code, value in ((code, cleaned_data[code]) for code in changed_data):
            obj.fields.create(code=code, value=library[code].to_database(value, obj.pk, code))


class ResourceAddForm(forms.ModelForm):
    class Meta:
        model = Resource
        fields = ('site', 'type', 'title', 'slug', 'parent', 'order')
        readonly = ('parent', )



from django.db import models
from django import test
from warthog import cache


class CacheTest(models.Model):
    code = models.CharField(max_length=50, unique=True)
    other = models.IntegerField()

    class Meta:
        app_label = 'warthog_test'


class CacheTestCase(test.TestCase):
    def setUp(self):
        cache.default_cache.clear()

    def test_generate_obj_key_with_instance(self):
        m = CacheTest(pk=1, code='foo', other=69)
        actual = cache.generate_obj_key(m, pk=m.pk)

        self.assertEquals('model:warthog_test.cachetest[pk=1]', actual)

    def test_generate_obj_key_with_type(self):
        actual = cache.generate_obj_key(CacheTest, pk=2)

        self.assertEquals('model:warthog_test.cachetest[pk=2]', actual)

    def test_set_model(self):
        m = CacheTest(pk=1, code='foo', other=69)

        key = cache.set_model(m)
        self.assertEqual('model:warthog_test.cachetest[pk=1]', key)

        actual = cache.default_cache.get(key)
        self.assertIsNotNone(actual)
        self.assertIsInstance(actual, CacheTest)

    def test_get_model(self):
        actual = cache.get_model(CacheTest, 1)
        self.assertIsNone(actual)

        # Arrange
        target = CacheTest(pk=1, code='foo', other=69)
        cache.default_cache.set('model:warthog_test.cachetest[pk=1]', target)

        # Act
        actual = cache.get_model(CacheTest, 1)
        self.assertIsNotNone(actual)

    def test_set_model_by_attribute(self):
        # Act
        m = CacheTest(pk=1, code='foo', other=69)
        ref_key = cache.set_model_by_attribute(m, 'code')

        # Assert
        self.assertEqual('model:warthog_test.cachetest[code=foo]', ref_key)
        key = cache.default_cache.get(ref_key)
        self.assertEqual('model:warthog_test.cachetest[pk=1]', key)
        actual = cache.default_cache.get(key)
        self.assertIsNotNone(actual)
        self.assertIsInstance(actual, CacheTest)

    def test_get_model_by_attribute(self):
        actual = cache.get_model_by_attribute(CacheTest, 'code', 'foo')
        self.assertIsNone(actual)

        # Arrange
        target = CacheTest(pk=1, code='foo', other=69)
        cache.default_cache.set('model:warthog_test.cachetest[code=foo]', 'model:warthog_test.cachetest[pk=1]')
        cache.default_cache.set('model:warthog_test.cachetest[pk=1]', target)

        # Act
        actual = cache.get_model_by_attribute(CacheTest, 'code', 'foo')
        self.assertIsNotNone(actual)
        self.assertIsInstance(actual, CacheTest)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .fields import (CharResourceField, BooleanResourceField, DateResourceField,
                     DateTimeResourceField, TextResourceField, TimeResourceField, HtmlResourceField, FileResourceField,
                     ImageResourceField)


class _Registry(object):
    def __init__(self):
        self.__default = CharResourceField()
        self.__fields = { # Pre-register internal fields.
            'char': self.__default,
            'bool': BooleanResourceField(),
            'date': DateResourceField(),
            'datetime': DateTimeResourceField(),
            'time': TimeResourceField(),
            'text': TextResourceField(),
            'html': HtmlResourceField(),
            'file': FileResourceField(),
            'image': ImageResourceField(),
        }

    def register(self, code, instance, default=False):
        if len(code) > 25:
            raise ValueError('code argument must be 25 characters or less.')
        if default:
            self.__default = instance
        self.__fields[code] = instance

    @property
    def choices(self):
        """
        Returns choice list that can be used in forms/models.
        """
        return [(k, v.label) for k, v in self.__fields.iteritems()]

    def __getitem__(self, code):
        return self.__fields.get(code, self.__default)


# Library instance.
library = _Registry()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.template import TemplateDoesNotExist
from .models import Template

try:
    from django.template.loaders.base import Loader as BaseLoader
except ImportError:
    from django.template.loader import BaseLoader


class CmsTemplateLoader(BaseLoader):
    """Load templates from CMS template store"""
    is_usable = True

    def load_template_source(self, template_name, template_dirs=None):
        display_name = 'warthog:%s' % template_name
        try:
            template = Template.objects.get(name__exact=template_name)
            return template.content, display_name
        except Template.DoesNotExist:
            pass
        raise TemplateDoesNotExist(template_name)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        for resource in orm.Resource.objects.all():
            if resource.parent is None:
                resource.slug = ''
            else:
                resource.slug = resource.uri_path[len(resource.parent.uri_path):].strip('/')
            resource.save()

    def backwards(self, orm):
        pass # Note required

    models = {
        'warthog.resource': {
            'Meta': {'ordering': "['order', 'uri_path', 'title']", 'object_name': 'Resource'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'hide_from_menu': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'menu_class': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'menu_title_raw': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '100'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['warthog.Resource']", 'null': 'True', 'on_delete': 'models.PROTECT', 'blank': 'True'}),
            'publish_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'resources'", 'to': "orm['warthog.ResourceType']"}),
            'unpublish_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'uri_path': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '500', 'blank': 'True'})
        },
        'warthog.resourcefield': {
            'Meta': {'unique_together': "(('resource', 'code'),)", 'object_name': 'ResourceField'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'resource': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'fields'", 'to': "orm['warthog.Resource']"}),
            'value': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'warthog.resourcetype': {
            'Meta': {'object_name': 'ResourceType'},
            'child_types': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['warthog.ResourceType']", 'symmetrical': 'False', 'blank': 'True'}),
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'default_template': ('django.db.models.fields.CharField', [], {'max_length': '500'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '500', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'warthog.resourcetypefield': {
            'Meta': {'unique_together': "(('resource_type', 'code'),)", 'object_name': 'ResourceTypeField'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'field_type': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'help_text': ('django.db.models.fields.CharField', [], {'max_length': '500', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label_raw': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'resource_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'fields'", 'to': "orm['warthog.ResourceType']"})
        },
        'warthog.template': {
            'Meta': {'ordering': "['name']", 'object_name': 'Template'},
            'cacheable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime_type': ('django.db.models.fields.CharField', [], {'default': "'text/html'", 'max_length': '25'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['warthog']
    symmetrical = True



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django import template
from ..models import Resource
from ..render import render_resource

register = template.Library()


@register.simple_tag(takes_context=True)
def inline_resource(context, pk_or_path, not_found='Resource `{}` not found'):
    """
    Render a resource inline

    :param context: Current render context
    :param pk_or_path: An ID or path of resource to be rendered
    :param not_found: Text to display if resource is not found.
    """
    try:
        request = context['request']
    except KeyError:
        raise KeyError('The request object is required to be part of the context. '
                       'Add "django.core.context_processors.request" to your TEMPLATE_CONTEXT_PROCESSORS setting')

    try:
        filters = dict(pk=int(pk_or_path))
    except ValueError:
        # Assume is path
        filters = dict(uri_path=pk_or_path)

    try:
        resource = Resource.objects.get_front(**filters)
    except Resource.DoesNotExist:
        return not_found.format(pk_or_path)
    else:
        if resource.can_serve(request.user):
            return render_resource(resource, request)

    return ''  # Return empty text if user permissions don't pass



# -*- coding: utf-8 -*-
from logging import getLogger
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.views.generic import View

from .models import Resource
from .render import render_resource


logger = getLogger('warthog.views')


class Cms(View):
    """View for displaying CMS resources.

    **Example**::

        from django.conf.urls import *
        from warthog.views import Cms

        urlpatterns = patterns('',
            url(r'^content/', Cms.as_view()),
        )

    The ``as_view`` method takes a several options:

    ``superuser_view_all``
        ``bool`` value to indicate that users with the ``is_superuser`` flag
        can view all pages including those which have not been published or
        are outside the published date range. **Default:** ``True``
    ``permission``
        Name of a permission that gives a user the ability to view all pages
        including those which have not been published or are outside the
        published date range. **Default:** ``preview_resource``

    .. note::
        This view is also used by :ref:CMSMiddleware to handle page requests.

    """
    def load_resource(self, *args, **kwargs):
        """Load the actual resource."""
        try:
            return Resource.objects.get_uri_path(self.request.path_info)
        except Resource.DoesNotExist:
            raise Http404

    @property
    def can_serve_flags(self):
        try:
            return self._can_serve_flags
        except AttributeError:
            flags = {}
            if hasattr(self, 'superuser_view_all'):
                flags['allow_superuser'] = getattr(self, 'superuser_view_all')
            if hasattr(self, 'permission'):
                flags['permission'] = getattr(self, 'permission')
            self._can_serve_flags = flags
            return flags

    def get(self, request, *args, **kwargs):
        """Respond to ``get`` HTTP method."""
        resource = self.load_resource(*args, **kwargs)

        if not resource.can_serve(request.user, **self.can_serve_flags):
            raise Http404

        return HttpResponse(render_resource(resource, request))


class CmsPreview(Cms):
    """View for previewing CMS resources.

    **Example**::

        from django.conf.urls import *
        from warthog.views import CmsPreview

        urlpatterns = patterns('',
            url(r'^preview/(\d+)/$', CmsPreview.as_view()),
        )

    The ``as_view`` method takes a several options:

    ``superuser_view_all``
        ``bool`` value to indicate that users with the ``is_superuser`` flag
        can view all pages including those which have not been published or
        are outside the published date range. **Default:** ``True``
    ``permission``
        Name of a permission that gives a user the ability to view all pages
        including those which have not been published or are outside the
        published date range. **Default:** ``preview_resource``

    """
    def load_resource(self, resource_id):
        return get_object_or_404(Resource, pk=resource_id)



# -*- coding: utf-8 -*-
from models import Resource

# TODO: A lot of opportunity here for caching


class ResourceItemFields(object):
    """
    Wrapper around resource item fields
    """
    __slots__ = ('__field_map', )

    def __init__(self, fields):
        field_lookup = {}
        for field in fields:
            field_lookup[field.code] = field.value
        self.__field_map = field_lookup

    def __getitem__(self, item):
        return self.__field_map[item]


class ResourceItem(object):
    """
    Wrapper around resource item
    """
    def __init__(self, resource):
        self.resource = resource
        self.vars = ResourceItemFields(resource.fields.all())

    @property
    def title(self):
        return self.resource.title

    @property
    def menu_title(self):
        return self.resource.menu_title

    @property
    def menu_class(self):
        return self.resource.menu_class

    @property
    def uri_path(self):
        return self.resource.uri_path


class ResourceIterator(object):
    """
    Resource iterator for iterating over resource query sets
    """
    def __init__(self, queryset):
        self.resources = queryset

    @classmethod
    def for_type(cls, resource_type, include_hidden=False):
        """
        Get a resource iterator for a particular resource type.
        :param resource_type: Resource type object.
        :return:
        """
        queryset = Resource.objects.all()
        if not include_hidden:
            queryset = queryset.filter(hide_from_menu=False)
        return cls(queryset)

    @classmethod
    def for_children(cls, resource, include_hidden=False):
        """
        Get a resource iterator for child objects.
        :param cls:
        :param resource:
        :return:
        """
        queryset = resource.children.all()
        if not include_hidden:
            queryset = queryset.filter(hide_from_menu=False)
        return cls(queryset)

    def __iter__(self):
        for resource in self.resources:
            if resource.is_live:
                yield ResourceItem(resource)

    def __len__(self):
        return len(self.resources)



from os import path
from setuptools import setup
import warthog

if warthog.VERSION[-1] in ('final', 0):
    CLASSIFIERS = ['Development Status :: 5 - Production/Stable']
elif 'beta' in warthog.VERSION[-1]:
    CLASSIFIERS = ['Development Status :: 4 - Beta']
else:
    CLASSIFIERS = ['Development Status :: 3 - Alpha']

CLASSIFIERS += [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

setup(
    author="Tim Savage",
    author_email="tim@savage.company",
    name='django-warthog',
    version=warthog.__version__,
    description='Embeddable CMS for Django.',
    long_description=open(path.join(path.dirname(__file__), 'README.rst')).read(),
    url='https://www.github.com/timsavage/django-warthog',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'Django>=1.4',
    ],
    packages=[
        'warthog',
        'warthog.admin',
        'warthog.migrations',
        'warthog.migrations_south',
        'warthog.templatetags',
        'warthog.resource_types',
        'warthog.tests',
    ],
    package_data={
        'warthog': [
            'static/admin/css/warthog.css',
            'static/admin/css/img/warthog-sprite.png',
            'templates/admin/warthog/resource_types/*.html',
            'templates/admin/warthog/*.html',
            'templates/warthog/*.html',
        ],
    },
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.conf.urls import url
from .views import CmsPreview

urlpatterns = [
    url(r'^_preview/(\d+)/$', CmsPreview.as_view(), name='warthog-preview'),
]



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Template'
        db.create_table('warthog_template', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100, db_index=True)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('mime_type', self.gf('django.db.models.fields.CharField')(default='text/html', max_length=25)),
            ('cacheable', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('warthog', ['Template'])

        # Adding model 'ResourceType'
        db.create_table('warthog_resourcetype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=50)),
            ('code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=50)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=500, blank=True)),
            ('default_template', self.gf('django.db.models.fields.CharField')(max_length=500)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('warthog', ['ResourceType'])

        # Adding model 'ResourceTypeField'
        db.create_table('warthog_resourcetypefield', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('resource_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='fields', to=orm['warthog.ResourceType'])),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('field_type', self.gf('django.db.models.fields.CharField')(max_length=25)),
            ('required', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('label_raw', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, blank=True)),
            ('help_text', self.gf('django.db.models.fields.CharField')(max_length=500, null=True, blank=True)),
        ))
        db.send_create_signal('warthog', ['ResourceTypeField'])

        # Adding unique constraint on 'ResourceTypeField', fields ['resource_type', 'code']
        db.create_unique('warthog_resourcetypefield', ['resource_type_id', 'code'])

        # Adding model 'Resource'
        db.create_table('warthog_resource', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'resources', to=orm['warthog.ResourceType'])),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('uri_path', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=500, blank=True)),
            ('published', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('publish_date', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('unpublish_date', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['warthog.Resource'], null=True, on_delete=models.PROTECT, blank=True)),
            ('menu_title_raw', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('menu_class', self.gf('django.db.models.fields.CharField')(max_length=50, blank=True)),
            ('hide_from_menu', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=100)),
            ('deleted', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('warthog', ['Resource'])

        # Adding unique constraint on 'Resource', fields ['uri_path', 'published']
        db.create_unique('warthog_resource', ['uri_path', 'published'])

        # Adding model 'ResourceField'
        db.create_table('warthog_resourcefield', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('resource', self.gf('django.db.models.fields.related.ForeignKey')(related_name='fields', to=orm['warthog.Resource'])),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('value', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('warthog', ['ResourceField'])

        # Adding unique constraint on 'ResourceField', fields ['resource', 'code']
        db.create_unique('warthog_resourcefield', ['resource_id', 'code'])


    def backwards(self, orm):
        # Removing unique constraint on 'ResourceField', fields ['resource', 'code']
        db.delete_unique('warthog_resourcefield', ['resource_id', 'code'])

        # Removing unique constraint on 'Resource', fields ['uri_path', 'published']
        db.delete_unique('warthog_resource', ['uri_path', 'published'])

        # Removing unique constraint on 'ResourceTypeField', fields ['resource_type', 'code']
        db.delete_unique('warthog_resourcetypefield', ['resource_type_id', 'code'])

        # Deleting model 'Template'
        db.delete_table('warthog_template')

        # Deleting model 'ResourceType'
        db.delete_table('warthog_resourcetype')

        # Deleting model 'ResourceTypeField'
        db.delete_table('warthog_resourcetypefield')

        # Deleting model 'Resource'
        db.delete_table('warthog_resource')

        # Deleting model 'ResourceField'
        db.delete_table('warthog_resourcefield')


    models = {
        'warthog.resource': {
            'Meta': {'ordering': "['order', 'uri_path', 'title']", 'unique_together': "(('uri_path', 'published'),)", 'object_name': 'Resource'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'hide_from_menu': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'menu_class': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'menu_title_raw': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '100'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['warthog.Resource']", 'null': 'True', 'on_delete': 'models.PROTECT', 'blank': 'True'}),
            'publish_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'resources'", 'to': "orm['warthog.ResourceType']"}),
            'unpublish_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'uri_path': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '500', 'blank': 'True'})
        },
        'warthog.resourcefield': {
            'Meta': {'unique_together': "(('resource', 'code'),)", 'object_name': 'ResourceField'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'resource': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'fields'", 'to': "orm['warthog.Resource']"}),
            'value': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'warthog.resourcetype': {
            'Meta': {'object_name': 'ResourceType'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'default_template': ('django.db.models.fields.CharField', [], {'max_length': '500'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '500', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'warthog.resourcetypefield': {
            'Meta': {'unique_together': "(('resource_type', 'code'),)", 'object_name': 'ResourceTypeField'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'field_type': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'help_text': ('django.db.models.fields.CharField', [], {'max_length': '500', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label_raw': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'resource_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'fields'", 'to': "orm['warthog.ResourceType']"})
        },
        'warthog.template': {
            'Meta': {'ordering': "['name']", 'object_name': 'Template'},
            'cacheable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime_type': ('django.db.models.fields.CharField', [], {'default': "'text/html'", 'max_length': '25'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['warthog']


# -*- coding: utf-8 -*-
from __future__ import absolute_import
import datetime
from django.contrib.admin import widgets as admin_widgets
from django.core import validators
from django.forms import fields
from ..resource_types import uploads

# Attempt Tinymce for HTML widget
try:
    from tinymce.widgets import AdminTinyMCE as AdminHtmlWidget
except ImportError:
    AdminHtmlWidget = admin_widgets.AdminTextareaWidget


class ResourceFieldBase(object):
    label = None
    field = fields.CharField
    widget = None

    def to_database(self, value, resource, code):
        """
        Convert an internal value into a string form for storage in the database.
        :param value: The value to be converted to a string.
        :param resource: resource this item is being saved to.
        :param code: Field code this item is being saved for.
        :return:
        """
        return value

    def to_python(self, value, resource, code):
        """
        Convert a value from the database into a value for use internally and in views.
        :param value: Value from the database.
        :param resource: resource this item is being loaded from.
        :param code: Field code this item is being loaded for.
        :return:
        """
        return value

    def create_form_field(self, field_kwargs, widget_kwargs=None):
        """
        Create an instance of a form field.
        :return:
        """
        field_kwargs = field_kwargs or dict()

        if self.widget:
            widget_kwargs = widget_kwargs or dict()
            field_kwargs['widget'] = self.widget(**widget_kwargs)

        return self.field(**field_kwargs)


class CharResourceField(ResourceFieldBase):
    label = 'Text'


class TextResourceField(ResourceFieldBase):
    label = 'Text Area'
    widget = admin_widgets.AdminTextareaWidget


class BooleanResourceField(ResourceFieldBase):
    label = "Checkbox"
    field = fields.BooleanField

    def to_database(self, value, resource, code):
        return 'true' if value else 'false'

    def to_python(self, value, resource, code):
        if isinstance(value, basestring) and value.lower() in ('false', '0'):
            return False
        else:
            return bool(value)


class TemporalResourceField(ResourceFieldBase):
    def to_python(self, value, resource, code):
        if value in validators.EMPTY_VALUES:
            return None
        return self.strptime(value)

    def strptime(self, value):
        raise NotImplementedError()


class DateResourceField(TemporalResourceField):
    label = 'Date'
    field = fields.DateField
    widget = admin_widgets.AdminDateWidget

    FORMAT = '%Y-%m-%d'

    def to_database(self, value, resource, code):
        if isinstance(value, (datetime.date, datetime.datetime)):
            return value.strftime(self.FORMAT)
        else:
            return None

    def strptime(self, value):
        return datetime.datetime.strptime(value, self.FORMAT).date()


class TimeResourceField(TemporalResourceField):
    label = 'Time'
    field = fields.TimeField
    widget = admin_widgets.AdminTimeWidget

    FORMAT = '%H:%M:%S'

    def to_database(self, value, resource, code):
        if isinstance(value, (datetime.time, datetime.datetime)):
            return value.strftime(self.FORMAT)
        else:
            return str(value)

    def strptime(self, value):
        return datetime.datetime.strptime(value, self.FORMAT).time()


class DateTimeResourceField(ResourceFieldBase):
    label = 'Date Time'
    field = fields.DateTimeField
    widget = admin_widgets.AdminSplitDateTime

    FORMAT = '%Y-%m-%dT%H:%M:%S'

    def to_database(self, value, resource, code):
        if isinstance(value, datetime.datetime):
            return value.strftime(self.FORMAT)
        else:
            return str(value)

    def to_python(self, value, resource, code):
        if value in validators.EMPTY_VALUES:
            return None
        return datetime.datetime.strptime(value, self.FORMAT)


class HtmlResourceField(ResourceFieldBase):
    """
    HTML resource_types
    """
    label = "HTML"
    widget = AdminHtmlWidget


class FileResourceField(ResourceFieldBase):
    label = "File"
    field = fields.FileField
    widget = admin_widgets.AdminFileWidget

    def to_database(self, value, resource, code):
        return uploads.save_file('resource_types/%s/%s-%s' % (resource.id, code, value.name), value)

    def to_python(self, value, resource, code):
        if value:
            return uploads.as_field_file(value)


class ImageResourceField(FileResourceField):
    label = "Image"
    field = fields.ImageField



# -*- coding: utf-8 -*-
from django.core.files.base import File
from django.core.files.storage import DefaultStorage


storage = DefaultStorage()


def save_file(name, uploaded_file):
    return storage.save(name, uploaded_file)


class FieldFile(File):
    def __init__(self, name, storage):
        super(FieldFile, self).__init__(None, name)
        self.storage = storage

    def __eq__(self, other):
        # Older code may be expecting FileField values to be simple strings.
        # By overriding the == operator, it can remain backwards compatibility.
        if hasattr(other, 'name'):
            return self.name == other.name
        return self.name == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        # Required because we defined a custom __eq__.
        return hash(self.name)

    def _get_file(self):
        if not hasattr(self, '_file') or self._file is None:
            self._file = self.storage.open(self.name, 'rb')
        return self._file

    def _set_file(self, file):
        self._file = file

    def _del_file(self):
        del self._file

    file = property(_get_file, _set_file, _del_file)

    def _get_path(self):
        return self.storage.path(self.name)
    path = property(_get_path)

    def _get_url(self):
        return self.storage.url(self.name)
    url = property(_get_url)


def as_field_file(name):
    return FieldFile(name, storage)
